@@ -1,22 +1,23 @@
-from obspy import UTCDateTime
-from obspy.geodetics import gps2dist_azimuth
 import re
-from seisloc.hypoinv import load_sum_evstr,load_sum_evid
 import os
 import glob
+import shutil
+import random
+from obspy import UTCDateTime
+from obspy.geodetics import gps2dist_azimuth
+from seisloc.hypoinv import load_sum_evstr,load_sum_evid
 from tqdm import tqdm
 import numpy as np
 import pandas as pd
 import matplotlib.pyplot as plt
 from seisloc.geometry import in_rectangle,loc_by_width
 from math import ceil,floor
-import shutil
-import random
 import multiprocessing as mp
 import time
 import subprocess
 import copy
-
+from PIL import Image
+#
 def load_PC(catalog="/home/zijinping/Desktop/projects/wy_eq/2018_2019_PC/2018_2019_hypoDD.reloc",
             start_evid=300000):
     """
@@ -58,13 +59,27 @@ def load_CEDC(catalog="/home/zijinping/Dropbox/resources/catalog/CEDC/20090101_2
 
 class DD():
     def __init__(self,reloc_file="hypoDD.reloc"):
+        """
+        The programme will read in hypoDD relocation file by default. If no hypoDD
+        file provided, it will generate an empty catalog. A user can set up a new 
+        catalog by providing a dict in the form:
+            dict[evid] = [lon,lat,dep,mag,UTCDateTime]
+        example:
+        >>> dd = DD()
+        >>> dd.dict = cata_dict  #cata_dict is a dictionary following above format
+        >>> dd.init()
+        >>> print(dd)
+        """
         try:
             self.dict,_ = load_DD(reloc_file)
+            print("successfully load catalog file: "+reloc_file)
             self.init()
         except:
             print("No hypoDD data read in, an empty catalog generated")
-            print("You can define self.dict[key] = [lon,lat,dep,mag,UTCDateTime]}")
+            print("You can define self.dict[evid] = [lon,lat,dep,mag,UTCDateTime]}")
             print("Then run: .init() function")
+            self.dict = {}
+
     def init(self):
         self.init_keys()
         self.init_locs()
@@ -354,10 +369,21 @@ class DD():
         plt.gca().set_aspect(aspect)
         plt.show()
     
-    def MT_plot(self,ref_time=UTCDateTime(2019,3,1),xlim=[],ylim=[0,5],unit="day",cmap=None,vmin=0,vmax=1):
+    def MT_plot(self,
+                ref_time=UTCDateTime(2019,3,1),
+                xlim=[],
+                ylim=[0,5],
+                unit="day",
+                cmap=None,
+                vmin=0,
+                vmax=1,
+                mlow=0,
+                plt_show=True,
+                figsize=(10,5)):
         """
         unit: 'day','hour' or 'second'
         """
+        fig,ax = plt.subplots(1,figsize=figsize)
         if unit == "day":
             denominator = (24*60*60)
             plt.xlabel("Time (day)")
@@ -374,17 +400,19 @@ class DD():
             diff_seconds = etime - ref_time
             diff_x = diff_seconds/denominator
             if cmap == None:
-                plt.plot([diff_x,diff_x],[0,emag],c='grey')
+                plt.plot([diff_x,diff_x],[mlow,emag],c='grey')
             else:
-                plt.plot([diff_x,diff_x],[0,emag],color=cmap((diff_x-vmin)/(vmax-vmin)))
+                plt.plot([diff_x,diff_x],[mlow,emag],color=cmap((diff_x-vmin)/(vmax-vmin)))
             plt.plot([diff_x],emag,'x',c='k')
         plt.ylim(ylim)
         if len(xlim)>0:
             plt.xlim(xlim)
         plt.ylabel("Magnitude")
-        plt.show()
+        if plt_show:
+            plt.show()
 
-    def dep_dist_plot(self,refid,
+    def dep_dist_plot(self,refid=None,
+                  refloc = [],
                   ref_time=UTCDateTime(2019,3,1),
                   xlim=[],
                   deplim=[],
@@ -415,9 +443,12 @@ class DD():
             axs[1].set_ylim(distlim)
         axs[0].grid(axis="y")
         axs[1].grid(axis="y")
-        reflon = self.dict[refid][0]
-        reflat = self.dict[refid][1]
-        refdep = self.dict[refid][2]
+        if refid != None:
+            reflon = self.dict[refid][0]
+            reflat = self.dict[refid][1]
+            refdep = self.dict[refid][2]
+        if len(refloc)>0:
+            reflon,reflat,refdep = refloc
         for evid in self.keys:
             etime =self.dict[evid][4]
             elon = self.dict[evid][0]
@@ -450,7 +481,7 @@ class DD():
         ax.set_ylim([depthmax,depthmin])
         plt.show()
         
-    def day_hist(self,ref_time=UTCDateTime(2019,1,1,0,0,0),ylim=[],color='b',edgecolor='k',plot_months=True,plt_show=True):
+    def day_hist(self,ref_time=UTCDateTime(2019,1,1,0,0,0),xlim=[],ylim=[],color='b',edgecolor='k',plot_months=True,plt_show=True,figsize=None):
         """
         Plot events by day-quantity in a histogram plot.
         Parameters:
@@ -465,11 +496,11 @@ class DD():
         min_day=floor(min(ref_list))
         max_day=ceil(max(ref_list))
         bins = np.linspace(min_day,max_day,max_day-min_day+1)
-        fig1 = plt.figure(1,figsize=(8,4))
+        if figsize==None:
+            figsize=(8,4)
+        fig1 = plt.figure(1,figsize=figsize)
         ax1 = plt.subplot(1,1,1)
         ax1.hist(ref_list,bins,color=color,edgecolor=edgecolor)
-        if ylim!=[]:
-            plt.ylim(ylim)
         # The bottom x-axis is in days
         ax1.set_xlim([0,max_day])
         # The top x-axis marks year and month in YYYYMM
@@ -501,11 +532,196 @@ class DD():
             ax2.plot(0,0,'k.')
             plt.xticks(tick_list_1,tick_list_2)
             ax2.set_xlabel("date")
+        if xlim!=[]:
+            ax1.set_xlim(xlim)
+            if plot_months:
+                ax2.set_xlim(xlim)
+        if ylim!=[]:
+            plt.ylim(ylim)
         ax1.set_xlabel("Time, days")
         ax1.set_ylabel("event quantity")
         if plt_show:
             plt.show()
 
+    def diffusion_plot(self,refid=None,refloc=[],diff_cfs=[],unit="day",xlim=[],ylim=[],plt_show=True):
+        '''
+        Parameters:
+        refid: reference event id
+        refloc: [lon,lat], reference site longitude and latitude, if not provided, use refid
+        diff_cfs: diffusion coefficient list, this will draw corresponding lines on the map
+        '''
+        if refid==None and refloc==[]:
+            raise Exception("refid or refloc should be proivded")
+        if refloc==[]:
+            refloc=[self.dict[refid][0],self.dict[refid][1]]
+        dist_list = np.zeros((len(self.keys),1))
+        day_list = np.zeros((len(self.keys),1))
+        mag_list = np.zeros((len(self.keys),1))
+        for i in range(len(self.keys)):
+            dist,_,_ = gps2dist_azimuth(self.locs[i,1],\
+                                self.locs[i,0],\
+                                refloc[1],\
+                                refloc[0])
+            day_list[i,0] = (self.relative_seconds[i]-np.min(self.relative_seconds))/(24*60*60)
+            mag_list[i,0] = self.locs[i,3]
+            dist_list[i,0] = dist
+
+        fig1 = plt.figure(1)
+        ax1 = plt.subplot(1,1,1)
+        if unit=="day":
+            x_list = day_list
+            plt.xlabel("Time (day)",fontsize=16)
+        elif unit == "hour":
+            x_list = day_list*24
+            plt.xlabel("Time (hour)",fontsize=16)
+        else:
+            raise Exception("Unit error: 'day' or 'hour'")
+        ax1.set_ylabel("Distance (m)",fontsize=16)
+        ax1.scatter(x_list,dist_list,(mag_list+2)*3,c='k')
+        ax1.set_xlim([0,np.max(x_list)])
+
+        diff_lines = []
+        if isinstance(diff_cfs,int) or isinstance(diff_cfs,float):
+            diff_cfs=[diff_cfs]
+        for diff_cf in diff_cfs:
+            if unit=="day":
+                x = np.linspace(0,np.max(x_list),int(np.max(x_list)*20)+1)
+                y = np.sqrt(4*np.pi*diff_cf*x*24*60*60)
+            elif unit=="hour":
+                x = np.linspace(0,np.max(x_list),int(np.max(x_list)*20)+1)
+                y = np.sqrt(4*np.pi*diff_cf*x*60*60)
+            diff_line, = plt.plot(x,y)
+            diff_lines.append(diff_line)
+
+        plt.legend(diff_lines,diff_cfs,title="Diffusion Coefficient $m^2/s$")
+        if len(xlim)>0:
+            plt.xlim(xlim)
+        if len(ylim)>0:
+            plt.ylim(ylim)
+        else:
+            plt.ylim(bottom=0)
+        if plt_show:
+            plt.show()
+
+    def animation(self,
+                  incre_hour=2,
+                  mb_time=None,
+                  me_time=None,
+                  xlim=[],
+                  ylim=[],
+                  geopara=None,
+                  cmap=None,
+                  vmin=None,
+                  vmax=None):
+        """
+        Generate gif animation file
+        increment: Time increased for each plot. Unit: hour
+        """
+        # Remove previous results
+        try:
+            shutil.rmtree("dd_animation")
+        except:
+            pass
+        os.makedirs("dd_animation")
+        if xlim == []:
+            xlim = [np.min(self.locs[:,0]),np.max(self.locs[:,0])]
+        if ylim == []:
+            ylim = [np.min(self.locs[:,1]),np.max(self.locs[:,1])]    
+        min_time = self.first_time+np.min(self.relative_seconds)
+        max_time = self.first_time+np.max(self.relative_seconds)
+        if mb_time == None:
+            mb_time = min_time
+        if me_time == None:
+            me_time = max_time
+        print("Movie start time is: ",mb_time)
+        print("  Movie end time is: ",me_time)
+
+        if vmin == None:
+            vmin = 0
+        if vmax == None:
+            vmax = (max_time - min_time)/(24*60*60)
+
+        inc_second = incre_hour*60*60 # Time increment
+        loop_time = mb_time
+        count = 1
+        ref_time = mb_time
+        while loop_time <= me_time:
+            fig = plt.figure(1,figsize=(8,8))
+            ax1 = fig.add_subplot(111)
+            ax1.set_xlim(xlim)
+            ax1.set_ylim(ylim)
+            ax1.set_xlabel("Lon(degree)",fontsize=18)
+            ax1.set_ylabel("Lat(degree)",fontsize=18)
+            ax1.set_title(f"{str(loop_time)[:19]}",fontsize=16)
+            if geopara != None:
+                # ----------------Molin faults--------------------------------
+                ml_fault = np.array(geopara.dict['ml_fault'])
+                ML_fault,=ax1.plot(ml_fault[:,0],ml_fault[:,1],'r-')
+                # ----------------Zigong faults-------------------------------
+                for key in geopara.dict['zg_faults']:
+                    array = np.array(geopara.dict['zg_faults'][key])
+                    ax1.plot(array[:,0],array[:,1],'k-',label='Faults')
+                #----------------------Wells-----------------------------------
+                wells = np.array(geopara.dict['wells'])
+                well_lons=[]; well_lats=[]
+                for well in geopara.dict['wells']:
+                    well_lons.append(well[0])
+                    well_lats.append(well[1])
+                s_well,=ax1.plot(well_lons,well_lats,'s',c='#1f77b4',markerfacecolor='white',mew=2,markersize=12)
+                #----------------------Stations--------------------------------
+                sta_lons=[]; sta_lats=[]
+                for sta in geopara.dict['sta_locs']:
+                    if sta[3]=="SC":
+                        sta_lons.append(sta[0]); sta_lats.append(sta[1])
+                if len(sta_lons)>0:
+                    s_sta=ax1.scatter(sta_lons,sta_lats,marker='^',c='cyan',s=120,edgecolor='k',label='Stations')
+            #------------- Events -----------------------------------------
+            eve_arr = []
+            rela_days = []
+            for i,second in enumerate(self.relative_seconds):
+                e_time = self.first_time + second
+                if e_time<(loop_time+inc_second/2) and e_time>mb_time:
+                    eve_arr.append(self.locs[i,:4])
+                    rela_days.append((e_time-ref_time)/(24*60*60))
+            eve_arr = np.array(eve_arr)
+            rela_days = np.array(rela_days)
+
+            if len(eve_arr)>0:
+                if cmap != None:
+                        s_eve=ax1.scatter(eve_arr[:,0],
+                                          eve_arr[:,1],
+                                          s=(eve_arr[:,3]+2)*5,
+                                          c=rela_days,
+                                          cmap=cmap,
+                                          vmin=vmin,
+                                          vmax=vmax,
+                                          label="Events")
+                else:
+                        s_eve=ax1.scatter(eve_arr[:,0],
+                                          eve_arr[:,1],
+                                          s=(eve_arr[:,3]+2)*5,
+                                          c='k',
+                                          label="Events")
+            if geopara != None:
+                plt.legend([s_well,s_sta,s_eve],\
+                       ["Platform",'Station',"Seismicity"],\
+                       loc='upper right',
+                       fontsize=16)  
+            ##------------- save results --------------------------------------------
+            plt.savefig(f"dd_animation/{str(count).zfill(3)}.png")
+            loop_time = loop_time + inc_second
+            count+=1
+            plt.close()
+        #-------------------- gif -----------------------------
+        imgs = []
+        for i in range(1,count):
+            pic_name = f'dd_animation/{str(i).zfill(3)}.png'
+            tmp = Image.open(pic_name)
+            imgs.append(tmp)
+        imgs[0].save("dd_animation.gif",save_all=True,append_images=imgs,duration=10)
+
+    def copy(self):
+        return copy.deepcopy(self)
     def merge(self,dd2):
         for key in dd2.keys:
             if key in self.keys:
@@ -513,9 +729,6 @@ class DD():
             self.dict[key]=dd2.dict[key]
         self.init()
 
-    def copy(self):
-        return copy.deepcopy(self)
-    
     def __repr__(self):
         _qty = f"HypoDD relocation catalog with {len(self.dict.keys())} events\n"
         _time= f"     Time range is: {self.first_time+np.min(self.relative_seconds)} to {self.first_time+np.max(self.relative_seconds)}\n"
@@ -662,6 +875,8 @@ def gen_dtcc(sta_list,sum_file="out.sum",work_dir="./",cc_threshold=0.7,min_link
             sta_pha = sta+"_"+pha
             globals()[sta_pha+"_cc_dict"] = {}                # Initiate dictionary
             sta_pha_path = os.path.join(work_dir,sta_pha)
+            if not os.path.exists(sta_pha_path):
+                print(sta_pha_path+" not exist!")
             for file in os.listdir(sta_pha_path):
                 if file[-3:]!=".xc":                          # none scc results file
                     continue
@@ -724,7 +939,7 @@ def gen_dtcc(sta_list,sum_file="out.sum",work_dir="./",cc_threshold=0.7,min_link
                 with open(cc_file,'a') as f:                # Write in results
                     f.write(f"# {format(evid1,'5d')} {format(evid2,'5d')} 0\n")
                     for record in link_cc:
-                        f.write(f"{record[0][:2]}{record[0]} {format(record[1],'7.4f')} {record[2]} {record[3]}\n")
+                        f.write(f"{record[0]} {format(record[1],'7.4f')} {record[2]} {record[3]}\n")
                 f.close()
     print(">>> Number of events in dt.cc is: ",len(to_cc_list))
     
@@ -833,7 +1048,7 @@ def load_DD(reloc_file="hypoDD.reloc",shift_hour=0):
 
     Parameters
     ----------
-    If the time of results is not in UTC time zone, a time shift is needed.
+    If the time of results is not in UTC time zone, a time shift might needed.
     For example, Beijing time zone is 8 hours early than UTC time, 8 hours 
     should be deducted so as to be consistent with UTC time.
     """
