@@ -275,3 +275,79 @@ def densityMap(lonlist,latlist,locs,longap,latgap,near=5):
                             if (i+kk)>=0 and (i+kk)<len(latlist) and (j+ll)>=0 and (j+ll)<len(lonlist):
                                 denSums[i+kk,j+ll]+=1
     return denCounts,denSums
+
+def rotate(xy,center,deg):
+    """
+    Degree is positive for anticlockwise
+    """
+    if isinstance(xy,list):
+        xy = np.array(xy)
+    if len(xy.shape)==1:
+        raise Error("xy should be 2 dimensional matrix")
+    if isinstance(center,list):
+        center = np.array(center)
+        
+    xy_ref = xy - center
+
+    rotate_matrix = [[np.cos(deg/180*pi),-np.sin(deg/180*pi)],[np.sin(deg/180*pi),np.cos(deg/180*pi)]]
+    rotate_matrix = np.array(rotate_matrix)
+
+    xy_rotate = np.matmul(rotate_matrix,xy_ref.T).T + center
+    
+    return xy_rotate
+
+def event_rotate(inFile,center,deg):
+    """
+    Rotate event input file for tomoDD
+    """
+    outFile = inFile+".rot"
+    cont = []
+    xys = []
+    with open(inFile,'r') as f:
+        for line in f:
+            line = line.rstrip()
+            cont.append(line)
+            _lat,_lon = line[20:39].split()
+            lat = float(_lat)
+            lon = float(_lon)
+            xys.append([lon,lat])       
+    xys_rot = rotate(xys,center=center,deg=deg)
+
+    f = open(outFile,'w')
+    for i in range(len(cont)):
+        line = cont[i]
+        f.write(line[:20])
+        f.write(format(xys_rot[i,1],">8.4f"))
+        f.write("  ")
+        f.write(format(xys_rot[i,0],">9.4f"))
+        f.write(line[39:])
+        f.write("\n")
+
+def sta_rotate(inFile,center,deg):
+    """
+    Rotate station file for tomoDD
+    """
+    outFile = inFile+".rot"
+    xys = []
+    stas = []
+    lonlats = []
+    _eles = []
+    with open(inFile,'r') as f:
+        for line in f:
+            line = line.rstrip()
+            sta,_lat,_lon,_ele = line.split()
+            stas.append(sta)
+            lonlats.append([float(_lon),float(_lat)])
+            _eles.append(_ele)
+            
+    lonlats_rot = rotate(lonlats,center=center,deg=deg)
+    f = open(outFile,'w')
+    for i in range(len(stas)):
+        f.write(format(stas[i],"<7s"))
+        f.write(" ")
+        f.write(format(lonlats_rot[i,1],'>10.6f'))
+        f.write(" ")
+        f.write(format(lonlats_rot[i,0],'>11.6f'))
+        f.write(" ")
+        f.write(_eles[i])
+        f.write("\n")
